@@ -33,6 +33,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#define _GNU_SOURCE
 #include <fcntl.h>
 #include <errno.h>
 #ifdef HAVE_GRP_H
@@ -505,6 +506,8 @@ get_xattrs (const char            *path,
 		g_free (escaped_attr);
 	      
 	      get_one_xattr (path, info, gio_attr, attr, follow_symlinks);
+
+	      g_free (gio_attr);
 	    }
 	      
 	  len = strlen (attr) + 1;
@@ -1234,8 +1237,13 @@ get_content_type (const char          *basename,
 	  sniff_length = _g_unix_content_type_get_sniff_len ();
 	  if (sniff_length > 4096)
 	    sniff_length = 4096;
-	  
-	  fd = open (path, O_RDONLY);
+
+#ifdef O_NOATIME	  
+          fd = open (path, O_RDONLY | O_NOATIME);
+          if (fd < 0 && errno == EPERM)
+#endif
+	    fd = open (path, O_RDONLY);
+
 	  if (fd != -1)
 	    {
 	      ssize_t res;
