@@ -42,13 +42,14 @@
 
 #undef G_DISABLE_DEPRECATED
 
-#include "galias.h"
 #include "glib.h"
 
 #include "giochannel.h"
 
 #include "glibintl.h"
 
+#include "galias.h"
+
 #define G_IO_NICE_BUF_SIZE	1024
 
 /* This needs to be as wide as the largest character in any possible encoding */
@@ -72,6 +73,16 @@ static GIOStatus	g_io_channel_read_line_backend	(GIOChannel  *channel,
 							 gsize       *terminator_pos,
 							 GError     **error);
 
+/**
+ * g_io_channel_init:
+ * @channel: a #GIOChannel
+ *
+ * Initializes a #GIOChannel struct. 
+ *
+ * This is called by each of the above functions when creating a 
+ * #GIOChannel, and so is not often needed by the application 
+ * programmer (unless you are creating a new type of #GIOChannel).
+ */
 void
 g_io_channel_init (GIOChannel *channel)
 {
@@ -91,21 +102,40 @@ g_io_channel_init (GIOChannel *channel)
   channel->close_on_unref = FALSE;
 }
 
-void 
+/**
+ * g_io_channel_ref:
+ * @channel: a #GIOChannel
+ *
+ * Increments the reference count of a #GIOChannel.
+ *
+ * Returns: the @channel that was passed in (since 2.6)
+ */
+GIOChannel *
 g_io_channel_ref (GIOChannel *channel)
 {
-  g_return_if_fail (channel != NULL);
+  g_return_val_if_fail (channel != NULL, NULL);
 
-  channel->ref_count++;
+  g_atomic_int_inc (&channel->ref_count);
+
+  return channel;
 }
 
+/**
+ * g_io_channel_unref:
+ * @channel: a #GIOChannel
+ *
+ * Decrements the reference count of a #GIOChannel.
+ */
 void 
 g_io_channel_unref (GIOChannel *channel)
 {
+  gboolean is_zero;
+
   g_return_if_fail (channel != NULL);
 
-  channel->ref_count--;
-  if (channel->ref_count == 0)
+  is_zero = g_atomic_int_dec_and_test (&channel->ref_count);
+
+  if (G_UNLIKELY (is_zero))
     {
       if (channel->close_on_unref)
         g_io_channel_shutdown (channel, TRUE, NULL);
@@ -116,8 +146,7 @@ g_io_channel_unref (GIOChannel *channel)
         g_iconv_close (channel->read_cd);
       if (channel->write_cd != (GIConv) -1)
         g_iconv_close (channel->write_cd);
-      if (channel->line_term)
-        g_free (channel->line_term);
+      g_free (channel->line_term);
       if (channel->read_buf)
         g_string_free (channel->read_buf, TRUE);
       if (channel->write_buf)
@@ -129,8 +158,8 @@ g_io_channel_unref (GIOChannel *channel)
 }
 
 static GIOError
-g_io_error_get_from_g_error (GIOStatus status,
-			     GError *err)
+g_io_error_get_from_g_error (GIOStatus  status,
+			     GError    *err)
 {
   switch (status)
     {
@@ -153,22 +182,22 @@ g_io_error_get_from_g_error (GIOStatus status,
           }
       default:
         g_assert_not_reached ();
-        return G_IO_ERROR_UNKNOWN; /* Keep the compiler happy */
     }
 }
 
 /**
  * g_io_channel_read:
- * @channel: a #GIOChannel. 
- * @buf: a buffer to read the data into (which should be at least count bytes long).
- * @count: the number of bytes to read from the #GIOChannel.
- * @bytes_read: returns the number of bytes actually read. 
+ * @channel: a #GIOChannel
+ * @buf: a buffer to read the data into (which should be at least 
+ *       count bytes long)
+ * @count: the number of bytes to read from the #GIOChannel
+ * @bytes_read: returns the number of bytes actually read
  * 
  * Reads data from a #GIOChannel. 
  * 
  * Return value: %G_IO_ERROR_NONE if the operation was successful. 
  *
- * Deprecated: Use g_io_channel_read_chars() instead.
+ * Deprecated:2.2: Use g_io_channel_read_chars() instead.
  **/
 GIOError 
 g_io_channel_read (GIOChannel *channel, 
@@ -204,16 +233,16 @@ g_io_channel_read (GIOChannel *channel,
 
 /**
  * g_io_channel_write:
- * @channel:  a #GIOChannel.
- * @buf: the buffer containing the data to write. 
- * @count: the number of bytes to write.
- * @bytes_written:  the number of bytes actually written.
+ * @channel:  a #GIOChannel
+ * @buf: the buffer containing the data to write
+ * @count: the number of bytes to write
+ * @bytes_written: the number of bytes actually written
  * 
  * Writes data to a #GIOChannel. 
  * 
  * Return value:  %G_IO_ERROR_NONE if the operation was successful.
  *
- * Deprecated: Use g_io_channel_write_chars() instead.
+ * Deprecated:2.2: Use g_io_channel_write_chars() instead.
  **/
 GIOError 
 g_io_channel_write (GIOChannel  *channel, 
@@ -240,23 +269,24 @@ g_io_channel_write (GIOChannel  *channel,
 
 /**
  * g_io_channel_seek:
- * @channel: a #GIOChannel. 
- * @offset: an offset, in bytes, which is added to the position specified by @type
+ * @channel: a #GIOChannel
+ * @offset: an offset, in bytes, which is added to the position specified 
+ *          by @type
  * @type: the position in the file, which can be %G_SEEK_CUR (the current
- *        position), %G_SEEK_SET (the start of the file), or %G_SEEK_END (the end of the
- *        file).
+ *        position), %G_SEEK_SET (the start of the file), or %G_SEEK_END 
+ *        (the end of the file)
  * 
- * Sets the current position in the #GIOChannel, similar to the standard library
- * function fseek(). 
+ * Sets the current position in the #GIOChannel, similar to the standard 
+ * library function fseek(). 
  * 
  * Return value: %G_IO_ERROR_NONE if the operation was successful.
  *
- * Deprecated: Use g_io_channel_seek_position() instead.
+ * Deprecated:2.2: Use g_io_channel_seek_position() instead.
  **/
 GIOError 
-g_io_channel_seek  (GIOChannel   *channel,
-		    gint64        offset, 
-		    GSeekType     type)
+g_io_channel_seek (GIOChannel *channel,
+		   gint64      offset, 
+		   GSeekType   type)
 {
   GError *err = NULL;
   GIOError error;
@@ -292,10 +322,10 @@ g_io_channel_seek  (GIOChannel   *channel,
 
 /**
  * g_io_channel_new_file:
- * @filename: A string containing the name of a file.
+ * @filename: A string containing the name of a file
  * @mode: One of "r", "w", "a", "r+", "w+", "a+". These have
- *        the same meaning as in fopen().
- * @error: A location to return an error of type %G_FILE_ERROR.
+ *        the same meaning as in fopen()
+ * @error: A location to return an error of type %G_FILE_ERROR
  *
  * Open a file @filename as a #GIOChannel using mode @mode. This
  * channel will be closed when the last reference to it is dropped,
@@ -314,7 +344,7 @@ g_io_channel_seek  (GIOChannel   *channel,
  * flushed, ignoring errors. The channel will not be freed until the
  * last reference is dropped using g_io_channel_unref(). 
  *
- * Deprecated: Use g_io_channel_shutdown() instead.
+ * Deprecated:2.2: Use g_io_channel_shutdown() instead.
  **/
 void
 g_io_channel_close (GIOChannel *channel)
@@ -352,9 +382,9 @@ g_io_channel_close (GIOChannel *channel)
  * Return value: the status of the operation.
  **/
 GIOStatus
-g_io_channel_shutdown (GIOChannel *channel,
-		       gboolean    flush,
-		       GError    **err)
+g_io_channel_shutdown (GIOChannel  *channel,
+		       gboolean     flush,
+		       GError     **err)
 {
   GIOStatus status, result;
   GError *tmperr = NULL;
@@ -458,15 +488,52 @@ g_io_channel_purge (GIOChannel *channel)
     }
 }
 
+/**
+ * g_io_create_watch:
+ * @channel: a #GIOChannel to watch
+ * @condition: conditions to watch for
+ *
+ * Creates a #GSource that's dispatched when @condition is met for the 
+ * given @channel. For example, if condition is #G_IO_IN, the source will 
+ * be dispatched when there's data available for reading.
+ *
+ * g_io_add_watch() is a simpler interface to this same functionality, for 
+ * the case where you want to add the source to the default main loop context 
+ * at the default priority.
+ *
+ * On Windows, polling a #GSource created to watch a channel for a socket
+ * puts the socket in non-blocking mode. This is a side-effect of the
+ * implementation and unavoidable.
+ *
+ * Returns: a new #GSource
+ */
 GSource *
-g_io_create_watch (GIOChannel  *channel,
-		   GIOCondition condition)
+g_io_create_watch (GIOChannel   *channel,
+		   GIOCondition  condition)
 {
   g_return_val_if_fail (channel != NULL, NULL);
 
   return channel->funcs->io_create_watch (channel, condition);
 }
 
+/**
+ * g_io_add_watch_full:
+ * @channel: a #GIOChannel
+ * @priority: the priority of the #GIOChannel source
+ * @condition: the condition to watch for
+ * @func: the function to call when the condition is satisfied
+ * @user_data: user data to pass to @func
+ * @notify: the function to call when the source is removed
+ *
+ * Adds the #GIOChannel into the default main loop context
+ * with the given priority.
+ *
+ * This internally creates a main loop source using g_io_create_watch()
+ * and attaches it to the main loop context with g_source_attach().
+ * You can do these steps manuallt if you need greater control.
+ *
+ * Returns: the event source id
+ */
 guint 
 g_io_add_watch_full (GIOChannel    *channel,
 		     gint           priority,
@@ -492,11 +559,23 @@ g_io_add_watch_full (GIOChannel    *channel,
   return id;
 }
 
+/**
+ * g_io_add_watch:
+ * @channel: a #GIOChannel
+ * @condition: the condition to watch for
+ * @func: the function to call when the condition is satisfied
+ * @user_data: user data to pass to @func
+ *
+ * Adds the #GIOChannel into the default main loop context
+ * with the default priority.
+ *
+ * Returns: the event source id
+ */
 guint 
-g_io_add_watch (GIOChannel    *channel,
-		GIOCondition   condition,
-		GIOFunc        func,
-		gpointer       user_data)
+g_io_add_watch (GIOChannel   *channel,
+		GIOCondition  condition,
+		GIOFunc       func,
+		gpointer      user_data)
 {
   return g_io_add_watch_full (channel, G_PRIORITY_DEFAULT, condition, func, user_data, NULL);
 }
@@ -506,9 +585,8 @@ g_io_add_watch (GIOChannel    *channel,
  * @channel: A #GIOChannel
  *
  * This function returns a #GIOCondition depending on whether there
- * is data to be read/space to write data in the
- * internal buffers in the #GIOChannel. Only the flags %G_IO_IN and
- * %G_IO_OUT may be set.
+ * is data to be read/space to write data in the internal buffers in 
+ * the #GIOChannel. Only the flags %G_IO_IN and %G_IO_OUT may be set.
  *
  * Return value: A #GIOCondition
  **/
@@ -536,11 +614,12 @@ g_io_channel_get_buffer_condition (GIOChannel *channel)
 
 /**
  * g_io_channel_error_from_errno:
- * @en: an <literal>errno</literal> error number, e.g. %EINVAL.
+ * @en: an <literal>errno</literal> error number, e.g. %EINVAL
  *
  * Converts an <literal>errno</literal> error number to a #GIOChannelError.
  *
- * Return value: a #GIOChannelError error number, e.g. %G_IO_CHANNEL_ERROR_INVAL.
+ * Return value: a #GIOChannelError error number, e.g. 
+ *      %G_IO_CHANNEL_ERROR_INVAL.
  **/
 GIOChannelError
 g_io_channel_error_from_errno (gint en)
@@ -623,13 +702,13 @@ g_io_channel_error_from_errno (gint en)
 /**
  * g_io_channel_set_buffer_size:
  * @channel: a #GIOChannel
- * @size: the size of the buffer. 0 == pick a good size
+ * @size: the size of the buffer, or 0 to let GLib pick a good size
  *
  * Sets the buffer size.
  **/  
 void
-g_io_channel_set_buffer_size (GIOChannel	*channel,
-                              gsize		 size)
+g_io_channel_set_buffer_size (GIOChannel *channel,
+                              gsize       size)
 {
   g_return_if_fail (channel != NULL);
 
@@ -651,7 +730,7 @@ g_io_channel_set_buffer_size (GIOChannel	*channel,
  * Return value: the size of the buffer.
  **/  
 gsize
-g_io_channel_get_buffer_size (GIOChannel	*channel)
+g_io_channel_get_buffer_size (GIOChannel *channel)
 {
   g_return_val_if_fail (channel != NULL, 0);
 
@@ -661,13 +740,13 @@ g_io_channel_get_buffer_size (GIOChannel	*channel)
 /**
  * g_io_channel_set_line_term:
  * @channel: a #GIOChannel
- * @line_term: The line termination string. Use %NULL for auto detect.
- *             Auto detection breaks on "\n", "\r\n", "\r", "\0", and
- *             the Unicode paragraph separator. Auto detection should
+ * @line_term: The line termination string. Use %NULL for autodetect.
+ *             Autodetection breaks on "\n", "\r\n", "\r", "\0", and
+ *             the Unicode paragraph separator. Autodetection should
  *             not be used for anything other than file-based channels.
  * @length: The length of the termination string. If -1 is passed, the
  *          string is assumed to be nul-terminated. This option allows
- *          termination strings with embeded nuls.
+ *          termination strings with embedded nuls.
  *
  * This sets the string that #GIOChannel uses to determine
  * where in the file a line break occurs.
@@ -685,8 +764,7 @@ g_io_channel_set_line_term (GIOChannel	*channel,
   else if (length < 0)
     length = strlen (line_term);
 
-  if (channel->line_term)
-    g_free (channel->line_term);
+  g_free (channel->line_term);
   channel->line_term = line_term ? g_memdup (line_term, length) : NULL;
   channel->line_term_len = length;
 }
@@ -698,16 +776,16 @@ g_io_channel_set_line_term (GIOChannel	*channel,
  *
  * This returns the string that #GIOChannel uses to determine
  * where in the file a line break occurs. A value of %NULL
- * indicates auto detection.
+ * indicates autodetection.
  *
  * Return value: The line termination string. This value
  *   is owned by GLib and must not be freed.
  **/
 G_CONST_RETURN gchar*
-g_io_channel_get_line_term (GIOChannel	*channel,
-			    gint        *length)
+g_io_channel_get_line_term (GIOChannel *channel,
+			    gint       *length)
 {
-  g_return_val_if_fail (channel != NULL, 0);
+  g_return_val_if_fail (channel != NULL, NULL);
 
   if (length)
     *length = channel->line_term_len;
@@ -717,24 +795,24 @@ g_io_channel_get_line_term (GIOChannel	*channel,
 
 /**
  * g_io_channel_set_flags:
- * @channel: a #GIOChannel.
- * @flags: the flags to set on the IO channel.
- * @error: A location to return an error of type #GIOChannelError.
+ * @channel: a #GIOChannel
+ * @flags: the flags to set on the IO channel
+ * @error: A location to return an error of type #GIOChannelError
  *
  * Sets the (writeable) flags in @channel to (@flags & %G_IO_CHANNEL_SET_MASK).
  *
  * Return value: the status of the operation. 
  **/
 GIOStatus
-g_io_channel_set_flags (GIOChannel *channel,
-                        GIOFlags    flags,
-                        GError    **error)
+g_io_channel_set_flags (GIOChannel  *channel,
+                        GIOFlags     flags,
+                        GError     **error)
 {
   g_return_val_if_fail (channel != NULL, G_IO_STATUS_ERROR);
   g_return_val_if_fail ((error == NULL) || (*error == NULL),
 			G_IO_STATUS_ERROR);
 
-  return (* channel->funcs->io_set_flags)(channel,
+  return (*channel->funcs->io_set_flags) (channel,
 					  flags & G_IO_FLAG_SET_MASK,
 					  error);
 }
@@ -750,7 +828,7 @@ g_io_channel_set_flags (GIOChannel *channel,
  * are cached for internal use by the channel when it is created.
  * If they should change at some later point (e.g. partial shutdown
  * of a socket with the UNIX shutdown() function), the user
- * should immediately call g_io_channel_get_flags () to update
+ * should immediately call g_io_channel_get_flags() to update
  * the internal values of these flags.
  *
  * Return value: the flags which are set on the channel
@@ -831,10 +909,10 @@ g_io_channel_get_close_on_unref	(GIOChannel *channel)
  * Return value: the status of the operation.
  **/
 GIOStatus
-g_io_channel_seek_position	(GIOChannel* channel,
-				 gint64      offset,
-				 GSeekType   type,
-				 GError    **error)
+g_io_channel_seek_position (GIOChannel  *channel,
+                            gint64       offset,
+                            GSeekType    type,
+                            GError     **error)
 {
   GIOStatus status;
 
@@ -986,8 +1064,8 @@ g_io_channel_flush (GIOChannel	*channel,
  * The default state of the channel is buffered.
  **/
 void
-g_io_channel_set_buffered	(GIOChannel *channel,
-				 gboolean    buffered)
+g_io_channel_set_buffered (GIOChannel *channel,
+                           gboolean    buffered)
 {
   g_return_if_fail (channel != NULL);
 
@@ -1006,14 +1084,14 @@ g_io_channel_set_buffered	(GIOChannel *channel,
 
 /**
  * g_io_channel_get_buffered:
- * @channel: a #GIOChannel.
+ * @channel: a #GIOChannel
  *
  * Returns whether @channel is buffered.
  *
  * Return Value: %TRUE if the @channel is buffered. 
  **/
 gboolean
-g_io_channel_get_buffered	(GIOChannel *channel)
+g_io_channel_get_buffered (GIOChannel *channel)
 {
   g_return_val_if_fail (channel != NULL, FALSE);
 
@@ -1024,43 +1102,51 @@ g_io_channel_get_buffered	(GIOChannel *channel)
  * g_io_channel_set_encoding:
  * @channel: a #GIOChannel
  * @encoding: the encoding type
- * @error: location to store an error of type #GConvertError.
+ * @error: location to store an error of type #GConvertError
  *
- * Sets the encoding for the input/output of the channel. The internal
- * encoding is always UTF-8. The default encoding for the
- * external file is UTF-8.
+ * Sets the encoding for the input/output of the channel. 
+ * The internal encoding is always UTF-8. The default encoding 
+ * for the external file is UTF-8.
  *
  * The encoding %NULL is safe to use with binary data.
  *
  * The encoding can only be set if one of the following conditions
  * is true:
- *
- * 1. The channel was just created, and has not been written to
- *    or read from yet.
- *
- * 2. The channel is write-only.
- *
- * 3. The channel is a file, and the file pointer was just
+ * <itemizedlist>
+ * <listitem><para>
+ *    The channel was just created, and has not been written to or read 
+ *    from yet.
+ * </para></listitem>
+ * <listitem><para>
+ *    The channel is write-only.
+ * </para></listitem>
+ * <listitem><para>
+ *    The channel is a file, and the file pointer was just
  *    repositioned by a call to g_io_channel_seek_position().
  *    (This flushes all the internal buffers.)
- *
- * 4. The current encoding is %NULL or UTF-8.
- *
- * 5. One of the (new API) read functions has just returned %G_IO_STATUS_EOF
- *    (or, in the case of g_io_channel_read_to_end (), %G_IO_STATUS_NORMAL).
- *
- * 6. One of the functions g_io_channel_read_chars () or g_io_channel_read_unichar ()
- *    has returned %G_IO_STATUS_AGAIN or %G_IO_STATUS_ERROR. This may be
- *    useful in the case of %G_CONVERT_ERROR_ILLEGAL_SEQUENCE.
- *    Returning one of these statuses from g_io_channel_read_line (),
- *    g_io_channel_read_line_string (), or g_io_channel_read_to_end ()
- *    does <emphasis>not</emphasis> guarantee that the encoding can be changed.
- *
+ * </para></listitem>
+ * <listitem><para>
+ *    The current encoding is %NULL or UTF-8.
+ * </para></listitem>
+ * <listitem><para>
+ *    One of the (new API) read functions has just returned %G_IO_STATUS_EOF
+ *    (or, in the case of g_io_channel_read_to_end(), %G_IO_STATUS_NORMAL).
+ * </para></listitem>
+ * <listitem><para>
+ *    One of the functions g_io_channel_read_chars() or 
+ *    g_io_channel_read_unichar() has returned %G_IO_STATUS_AGAIN or 
+ *    %G_IO_STATUS_ERROR. This may be useful in the case of 
+ *    %G_CONVERT_ERROR_ILLEGAL_SEQUENCE.
+ *    Returning one of these statuses from g_io_channel_read_line(),
+ *    g_io_channel_read_line_string(), or g_io_channel_read_to_end()
+ *    does <emphasis>not</emphasis> guarantee that the encoding can 
+ *    be changed.
+ * </para></listitem>
+ * </itemizedlist>
  * Channels which do not meet one of the above conditions cannot call
- * g_io_channel_seek_position () with an offset of %G_SEEK_CUR,
- * and, if they are "seekable", cannot
- * call g_io_channel_write_chars () after calling one
- * of the API "read" functions.
+ * g_io_channel_seek_position() with an offset of %G_SEEK_CUR, and, if 
+ * they are "seekable", cannot call g_io_channel_write_chars() after 
+ * calling one of the API "read" functions.
  *
  * Return Value: %G_IO_STATUS_NORMAL if the encoding was successfully set.
  **/
@@ -1113,8 +1199,8 @@ g_io_channel_set_encoding (GIOChannel	*channel,
           if (read_cd == (GIConv) -1)
             {
               err = errno;
-              from_enc = "UTF-8";
-              to_enc = encoding;
+              from_enc = encoding;
+              to_enc = "UTF-8";
             }
         }
       else
@@ -1127,8 +1213,8 @@ g_io_channel_set_encoding (GIOChannel	*channel,
           if (write_cd == (GIConv) -1)
             {
               err = errno;
-              from_enc = encoding;
-              to_enc = "UTF-8";
+              from_enc = "UTF-8";
+              to_enc = encoding;
             }
         }
       else
@@ -1141,11 +1227,11 @@ g_io_channel_set_encoding (GIOChannel	*channel,
 
           if (err == EINVAL)
             g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_NO_CONVERSION,
-                         _("Conversion from character set `%s' to `%s' is not supported"),
+                         _("Conversion from character set '%s' to '%s' is not supported"),
                          from_enc, to_enc);
           else
             g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-                         _("Could not open converter from `%s' to `%s': %s"),
+                         _("Could not open converter from '%s' to '%s': %s"),
                          from_enc, to_enc, g_strerror (err));
 
           if (read_cd != (GIConv) -1)
@@ -1192,15 +1278,15 @@ g_io_channel_set_encoding (GIOChannel	*channel,
  * g_io_channel_get_encoding:
  * @channel: a #GIOChannel
  *
- * Gets the encoding for the input/output of the channel. The internal
- * encoding is always UTF-8. The encoding %NULL makes the
- * channel safe for binary data.
+ * Gets the encoding for the input/output of the channel. 
+ * The internal encoding is always UTF-8. The encoding %NULL 
+ * makes the channel safe for binary data.
  *
  * Return value: A string containing the encoding, this string is
  *   owned by GLib and must not be freed.
  **/
 G_CONST_RETURN gchar*
-g_io_channel_get_encoding (GIOChannel      *channel)
+g_io_channel_get_encoding (GIOChannel *channel)
 {
   g_return_val_if_fail (channel != NULL, NULL);
 
@@ -1208,8 +1294,8 @@ g_io_channel_get_encoding (GIOChannel      *channel)
 }
 
 static GIOStatus
-g_io_channel_fill_buffer (GIOChannel *channel,
-                          GError    **err)
+g_io_channel_fill_buffer (GIOChannel  *channel,
+                          GError     **err)
 {
   gsize read_size, cur_len, oldlen;
   GIOStatus status;
@@ -1240,8 +1326,8 @@ g_io_channel_fill_buffer (GIOChannel *channel,
 
   g_string_truncate (channel->read_buf, read_size + cur_len);
 
-  if ((status != G_IO_STATUS_NORMAL)
-    && ((status != G_IO_STATUS_EOF) || (channel->read_buf->len == 0)))
+  if ((status != G_IO_STATUS_NORMAL) &&
+      ((status != G_IO_STATUS_EOF) || (channel->read_buf->len == 0)))
     return status;
 
   g_assert (channel->read_buf->len > 0);
@@ -1257,7 +1343,7 @@ g_io_channel_fill_buffer (GIOChannel *channel,
 
   if (channel->do_encode)
     {
-      size_t errnum, inbytes_left, outbytes_left;
+      gsize errnum, inbytes_left, outbytes_left;
       gchar *inbuf, *outbuf;
       int errval;
 
@@ -1291,7 +1377,7 @@ reencode:
       g_string_truncate (channel->encoded_read_buf,
 			 channel->encoded_read_buf->len - outbytes_left);
 
-      if (errnum == (size_t) -1)
+      if (errnum == (gsize) -1)
         {
           switch (errval)
             {
@@ -1311,7 +1397,7 @@ reencode:
                   status = G_IO_STATUS_NORMAL;
                 else
                   {
-                    g_set_error (err, G_CONVERT_ERROR,
+                    g_set_error_literal (err, G_CONVERT_ERROR,
                       G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
                       _("Invalid byte sequence in conversion input"));
                     return G_IO_STATUS_ERROR;
@@ -1353,7 +1439,7 @@ reencode:
                   status = G_IO_STATUS_NORMAL;
                 else
                   {
-                    g_set_error (err, G_CONVERT_ERROR,
+                    g_set_error_literal (err, G_CONVERT_ERROR,
                       G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
                       _("Invalid byte sequence in conversion input"));
                     status = G_IO_STATUS_ERROR;
@@ -1399,11 +1485,11 @@ reencode:
  * Return value: the status of the operation.
  **/
 GIOStatus
-g_io_channel_read_line (GIOChannel *channel,
-                        gchar     **str_return,
-                        gsize      *length,
-			gsize      *terminator_pos,
-		        GError    **error)
+g_io_channel_read_line (GIOChannel  *channel,
+                        gchar      **str_return,
+                        gsize       *length,
+			gsize       *terminator_pos,
+		        GError     **error)
 {
   GIOStatus status;
   gsize got_length;
@@ -1446,10 +1532,10 @@ g_io_channel_read_line (GIOChannel *channel,
  * Return value: the status of the operation.
  **/
 GIOStatus
-g_io_channel_read_line_string (GIOChannel *channel,
-                               GString	  *buffer,
-			       gsize      *terminator_pos,
-                               GError	 **error)
+g_io_channel_read_line_string (GIOChannel  *channel,
+                               GString	   *buffer,
+			       gsize       *terminator_pos,
+                               GError	  **error)
 {
   gsize length;
   GIOStatus status;
@@ -1477,10 +1563,10 @@ g_io_channel_read_line_string (GIOChannel *channel,
 
 
 static GIOStatus
-g_io_channel_read_line_backend	(GIOChannel *channel,
-				 gsize      *length,
-				 gsize      *terminator_pos,
-				 GError    **error)
+g_io_channel_read_line_backend (GIOChannel  *channel,
+                                gsize       *length,
+                                gsize       *terminator_pos,
+                                GError     **error)
 {
   GIOStatus status;
   gsize checked_to, line_term_len, line_length, got_term_len;
@@ -1489,8 +1575,8 @@ g_io_channel_read_line_backend	(GIOChannel *channel,
   if (!channel->use_buffer)
     {
       /* Can't do a raw read in read_line */
-      g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-                   _("Can't do a raw read in g_io_channel_read_line_string"));
+      g_set_error_literal (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
+                           _("Can't do a raw read in g_io_channel_read_line_string"));
       return G_IO_STATUS_ERROR;
     }
 
@@ -1535,9 +1621,10 @@ read_again:
 
                     if (channel->encoding && channel->read_buf->len != 0)
                       {
-                        g_set_error (error, G_CONVERT_ERROR,
-                                     G_CONVERT_ERROR_PARTIAL_INPUT,
-                                     _("Leftover unconverted data in read buffer"));
+                        g_set_error_literal (error, G_CONVERT_ERROR,
+                                             G_CONVERT_ERROR_PARTIAL_INPUT,
+                                             _("Leftover unconverted data in "
+                                               "read buffer"));
                         return G_IO_STATUS_ERROR;
                       }
                     else
@@ -1616,8 +1703,8 @@ read_again:
         {
           if (channel->encoding && channel->read_buf->len > 0)
             {
-              g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_PARTIAL_INPUT,
-                           _("Channel terminates in a partial character"));
+              g_set_error_literal (error, G_CONVERT_ERROR, G_CONVERT_ERROR_PARTIAL_INPUT,
+                                   _("Channel terminates in a partial character"));
               return G_IO_STATUS_ERROR;
             }
           line_length = use_buf->len;
@@ -1650,20 +1737,20 @@ done:
  *              be freed with g_free() when no longer needed. This
  *              data is terminated by an extra nul character, but there 
  *              may be other nuls in the intervening data.
- * @length: Location to store length of the data
- * @error: A location to return an error of type #GConvertError
+ * @length: location to store length of the data
+ * @error: location to return an error of type #GConvertError
  *         or #GIOChannelError
  *
  * Reads all the remaining data from the file.
  *
  * Return value: %G_IO_STATUS_NORMAL on success. 
- * This function never returns %G_IO_STATUS_EOF.
+ *     This function never returns %G_IO_STATUS_EOF.
  **/
 GIOStatus
-g_io_channel_read_to_end (GIOChannel	*channel,
-                          gchar        **str_return,
-                          gsize		*length,
-                          GError       **error)
+g_io_channel_read_to_end (GIOChannel  *channel,
+                          gchar      **str_return,
+                          gsize	      *length,
+                          GError     **error)
 {
   GIOStatus status;
     
@@ -1679,8 +1766,8 @@ g_io_channel_read_to_end (GIOChannel	*channel,
 
   if (!channel->use_buffer)
     {
-      g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-                   _("Can't do a raw read in g_io_channel_read_to_end"));
+      g_set_error_literal (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
+                           _("Can't do a raw read in g_io_channel_read_to_end"));
       return G_IO_STATUS_ERROR;
     }
 
@@ -1693,8 +1780,8 @@ g_io_channel_read_to_end (GIOChannel	*channel,
 
   if (channel->encoding && channel->read_buf->len > 0)
     {
-      g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_PARTIAL_INPUT,
-                   _("Channel terminates in a partial character"));
+      g_set_error_literal (error, G_CONVERT_ERROR, G_CONVERT_ERROR_PARTIAL_INPUT,
+                           _("Channel terminates in a partial character"));
       return G_IO_STATUS_ERROR;
     }
 
@@ -1735,7 +1822,7 @@ g_io_channel_read_to_end (GIOChannel	*channel,
  *              success if count < 6 and the channel's encoding is non-%NULL.
  *              This indicates that the next UTF-8 character is too wide for
  *              the buffer.
- * @error: A location to return an error of type #GConvertError
+ * @error: a location to return an error of type #GConvertError
  *         or #GIOChannelError.
  *
  * Replacement for g_io_channel_read() with the new API.
@@ -1743,11 +1830,11 @@ g_io_channel_read_to_end (GIOChannel	*channel,
  * Return value: the status of the operation.
  **/
 GIOStatus
-g_io_channel_read_chars (GIOChannel	*channel,
-                         gchar		*buf,
-                         gsize		 count,
-			 gsize          *bytes_read,
-                         GError        **error)
+g_io_channel_read_chars (GIOChannel  *channel,
+                         gchar	     *buf,
+                         gsize	      count,
+			 gsize       *bytes_read,
+                         GError     **error)
 {
   GIOStatus status;
   gsize got_bytes;
@@ -1792,9 +1879,9 @@ g_io_channel_read_chars (GIOChannel	*channel,
       if (status == G_IO_STATUS_EOF && channel->encoding
           && BUF_LEN (channel->read_buf) > 0)
         {
-          g_set_error (error, G_CONVERT_ERROR,
-                       G_CONVERT_ERROR_PARTIAL_INPUT,
-                       _("Leftover unconverted data in read buffer"));
+          g_set_error_literal (error, G_CONVERT_ERROR,
+                               G_CONVERT_ERROR_PARTIAL_INPUT,
+                               _("Leftover unconverted data in read buffer"));
           status = G_IO_STATUS_ERROR;
         }
 
@@ -1847,17 +1934,18 @@ g_io_channel_read_chars (GIOChannel	*channel,
  * g_io_channel_read_unichar:
  * @channel: a #GIOChannel
  * @thechar: a location to return a character
- * @error: A location to return an error of type #GConvertError
+ * @error: a location to return an error of type #GConvertError
  *         or #GIOChannelError
  *
+ * Reads a Unicode character from @channel.
  * This function cannot be called on a channel with %NULL encoding.
  *
  * Return value: a #GIOStatus
  **/
 GIOStatus
-g_io_channel_read_unichar     (GIOChannel   *channel,
-			       gunichar     *thechar,
-			       GError      **error)
+g_io_channel_read_unichar (GIOChannel  *channel,
+			   gunichar    *thechar,
+			   GError     **error)
 {
   GIOStatus status = G_IO_STATUS_NORMAL;
 
@@ -1878,9 +1966,9 @@ g_io_channel_read_unichar     (GIOChannel   *channel,
 
       if (status == G_IO_STATUS_EOF && BUF_LEN (channel->read_buf) > 0)
         {
-          g_set_error (error, G_CONVERT_ERROR,
-                       G_CONVERT_ERROR_PARTIAL_INPUT,
-                       _("Leftover unconverted data in read buffer"));
+          g_set_error_literal (error, G_CONVERT_ERROR,
+                               G_CONVERT_ERROR_PARTIAL_INPUT,
+                               _("Leftover unconverted data in read buffer"));
           status = G_IO_STATUS_ERROR;
         }
 
@@ -1914,7 +2002,7 @@ g_io_channel_read_unichar     (GIOChannel   *channel,
  *                 If the return value is %G_IO_STATUS_NORMAL and the
  *                 channel is blocking, this will always be equal
  *                 to @count if @count >= 0.
- * @error: A location to return an error of type #GConvertError
+ * @error: a location to return an error of type #GConvertError
  *         or #GIOChannelError
  *
  * Replacement for g_io_channel_write() with the new API.
@@ -1927,11 +2015,11 @@ g_io_channel_read_unichar     (GIOChannel   *channel,
  * Return value: the status of the operation.
  **/
 GIOStatus
-g_io_channel_write_chars (GIOChannel	*channel,
-                          const gchar	*buf,
-                          gssize	 count,
-			  gsize         *bytes_written,
-                          GError       **error)
+g_io_channel_write_chars (GIOChannel   *channel,
+                          const gchar  *buf,
+                          gssize        count,
+			  gsize        *bytes_written,
+                          GError      **error)
 {
   GIOStatus status;
   gssize wrote_bytes = 0;
@@ -2003,7 +2091,7 @@ g_io_channel_write_chars (GIOChannel	*channel,
        * and never receiving an EAGAIN.
        */
 
-      if (channel->write_buf->len >= channel->buf_size)
+      if (channel->write_buf->len >= channel->buf_size - MAX_CHAR_SIZE)
         {
           gsize did_write = 0, this_time;
 
@@ -2049,7 +2137,7 @@ g_io_channel_write_chars (GIOChannel	*channel,
         {
           const gchar *from_buf;
           gsize from_buf_len, from_buf_old_len, left_len;
-          size_t err;
+          gsize err;
           gint errnum;
 
           if (channel->partial_write_buf[0] != '\0')
@@ -2096,29 +2184,29 @@ reconvert:
                         if (try_len == from_buf_len)
                           {
                             errnum = EINVAL;
-                            err = (size_t) -1;
+                            err = (gsize) -1;
                           }
                         else
                           {
                             errnum = 0;
-                            err = (size_t) 0;
+                            err = (gsize) 0;
                           }
                         break;
                       case -1:
                         g_warning ("Invalid UTF-8 passed to g_io_channel_write_chars().");
                         /* FIXME bail here? */
                         errnum = EILSEQ;
-                        err = (size_t) -1;
+                        err = (gsize) -1;
                         break;
                       default:
                         g_assert_not_reached ();
-                        err = (size_t) -1;
+                        err = (gsize) -1;
                         errnum = 0; /* Don't confunse the compiler */
                     }
                 }
               else
                 {
-                  err = (size_t) 0;
+                  err = (gsize) 0;
                   errnum = 0;
                   left_len = from_buf_len - try_len;
                 }
@@ -2143,7 +2231,7 @@ reconvert:
                                   - space_in_buf);
             }
 
-          if (err == (size_t) -1)
+          if (err == (gsize) -1)
             {
               switch (errnum)
         	{
@@ -2195,7 +2283,7 @@ reconvert:
                       }
                     break;
                   case EILSEQ:
-                    g_set_error (error, G_CONVERT_ERROR,
+                    g_set_error_literal (error, G_CONVERT_ERROR,
                       G_CONVERT_ERROR_ILLEGAL_SEQUENCE,
                       _("Invalid byte sequence in conversion input"));
                     if (from_buf_old_len > 0 && from_buf_len == left_len)
@@ -2245,17 +2333,18 @@ reconvert:
  * g_io_channel_write_unichar:
  * @channel: a #GIOChannel
  * @thechar: a character
- * @error: A location to return an error of type #GConvertError
+ * @error: location to return an error of type #GConvertError
  *         or #GIOChannelError
  *
+ * Writes a Unicode character to @channel.
  * This function cannot be called on a channel with %NULL encoding.
  *
  * Return value: a #GIOStatus
  **/
 GIOStatus
-g_io_channel_write_unichar    (GIOChannel   *channel,
-			       gunichar      thechar,
-			       GError      **error)
+g_io_channel_write_unichar (GIOChannel  *channel,
+			    gunichar     thechar,
+			    GError     **error)
 {
   GIOStatus status;
   gchar static_buf[6];
@@ -2288,14 +2377,13 @@ g_io_channel_write_unichar    (GIOChannel   *channel,
 /**
  * g_io_channel_error_quark:
  *
- * Return value: The quark used as %G_IO_CHANNEL_ERROR
+ * Return value: the quark used as %G_IO_CHANNEL_ERROR
  **/
 GQuark
 g_io_channel_error_quark (void)
 {
-  static GQuark q = 0;
-  if (q == 0)
-    q = g_quark_from_static_string ("g-io-channel-error-quark");
-
-  return q;
+  return g_quark_from_static_string ("g-io-channel-error-quark");
 }
+
+#define __G_IOCHANNEL_C__
+#include "galiasdef.c"
